import configparser
import os
import re
from tkinter import filedialog
from .utils import port_is_used,get_host_ip

filepath = os.path.join(os.path.dirname(__file__), 'config.ini')
config = configparser.ConfigParser()
VERSION = '1.1.0'
HOST = 'https://www.shortvideo.work'

def save_config(section,key,value):
    config.read(filepath)
    if not config.has_section(section):
        config.add_section(section)
    config.set(section, key, value)
    config.write(open(filepath, "w"))

def read():
    if not os.path.exists(filepath):
        config.add_section("video")
        config.set("video", "version", VERSION)
        config.set("video", "desc", "请不在擅自修改配置文件，以免引发异常！")
        config.write(open(filepath, "w"))

    config.read(filepath)

    if not config.has_section('server'):
        config.add_section("server")
        config.set("server", "host", HOST)
        config.write(open(filepath, "w"))

    version = config.get('video', 'version', fallback=VERSION)
    if version!=VERSION:
        version = VERSION
        config.set('video', 'version', VERSION)
        config.write(open(filepath, "w"))

    host = config.get('server', 'host', fallback=HOST)
    appid = config.get('server', 'appid', fallback='')
    appsecret = config.get('server', 'appsecret', fallback='')

    while not appid:
        value = input('请输入appid：')
        if value:
            appid = value
            config.set('server', 'appid', value)
            config.write(open(filepath, "w"))
            break

    while not appsecret:
        value = input('请输入appsecret：')
        if value:
            appsecret = value
            config.set('server', 'appsecret', value)
            config.write(open(filepath, "w"))
            break


    return version, host, appid, appsecret

def read_workdir(key:str):
    if key not in ['publish','export','web']:
        print('无效的类型')
        return
    
    config.read(filepath)
    if not config.has_section(key):
        config.add_section(key)
    workdir = config.get(key, 'workdir', fallback='')
    text = {
        'publish':'请选择一个存放发布视频时的缓存目录（建议使用一个剩余空间较大的分区）',
        'export':'请选择一个存放导出视频和封面的目录（建议使用一个剩余空间较大的分区）',
        'web':'请选择一个存放上传视频和封面的目录（建议使用一个剩余空间较大的分区）'
    }
    while not workdir:
        if input(f'\n{text[key]}\n请按 y 键并回车继续：').lower() == 'y':
            value = filedialog.askdirectory()
            if value:
                workdir = value
                config.set(key, 'workdir', value)
                config.write(open(filepath, "w"))
                break
        else:
            break
    return workdir

def read_web():
    if not os.path.exists(filepath):
        config.add_section("video")
        config.set("video", "version", VERSION)
        config.set("video", "desc", "请不在擅自修改配置文件，以免引发异常！")
        config.write(open(filepath, "w"))

    config.read(filepath)
    if not config.has_section('web'):
        config.add_section("web")
        config.write(open(filepath, "w"))

    version = config.get('video', 'version', fallback=VERSION)
    port = config.getint('web', 'port', fallback=None)
    host = get_host_ip()
    while not port:
        value = input('请输入端口号：')
        if value and re.fullmatch('\d+',value):
            if port_is_used(host,int(value)):
                print(f'{value}端口已被占用')
                continue

            port = int(value)
            config.set('web', 'port', value)
            config.write(open(filepath, "w"))
            break

    workdir = read_workdir('web')
    return version,host, port, workdir


from typing import Match
from json import dumps
from datetime import datetime
from decimal import Decimal
from enum import Enum
from pathlib import Path

class Object():
    @staticmethod
    def default(obj: "Object"):
        if isinstance(obj, bytes):
            return repr(obj)

        if isinstance(obj, Match):
            return repr(obj)

        if isinstance(obj, Enum):
            return str(obj)

        if isinstance(obj,(datetime,Decimal)):
            return str(obj)

        if isinstance(obj,Path):
            return str(obj.absolute())

        return {
            "_": obj.__class__.__name__,
            **{
                attr: getattr(obj, attr)
                for attr in filter(lambda x: not x.startswith("_"), obj.__dict__)
            }
        }

    def __str__(self) -> str:
        return dumps(self, indent=4, default=Object.default, ensure_ascii=False)


import argparse
import asyncio
import datetime
import json
import logging
import os
import signal
import sys

import requests

from .config import read, read_workdir, save_config
from ._message import Message
from ._result import Result
from ._youban import Youban
from .utils import download, md5_value, sign_dict, to_cos

log = logging.getLogger(__name__)


class Publish:
    do_action: str

    def __init__(self, version: str, host: str, workdir: str, appid: str, appsecret: str) -> None:
        self.do_action = ''
        self.version = version
        self.host = host
        self.workdir = workdir
        self.appid = appid
        self.appsecret = appsecret

    async def run(self) -> Result:
        res = await self.query('/publish/read')
        if(not res.success or not res.data):
            return res

        await self.start(res.data)
        return Result(True, wait=0)

    async def start(self, data: dict):
        message = Message(**data)

        message.console_title()

        if message.category.mode not in ['web', 'api']:
            error = f'{message.category.mode}无效的发布类型'
            print(f'\n\t{error}')
            await self.update(message._id, 'completed', 'error', message=error)
            return

        start_seconds = datetime.datetime.now().timestamp()
        # 下载失败则直接中止
        if not await self.download(message):
            return

        if datetime.datetime.now().timestamp()-start_seconds < 2:
            await asyncio.sleep(5)

        if(message.category.mode == 'web'):
            await self.web_publish(message)
        elif message.category.mode == 'api':
            await self.api_publish(message)

    def remove_local_files(self, message: Message):
        for f in message.files:
            if f.local_path and os.path.exists(f.local_path):
                os.remove(f.local_path)

    async def download(self, message: Message):
        savepath = os.path.join(self.workdir, 'cache')
        os.makedirs(savepath, exist_ok=True)

        error = ''
        for file in message.files:
            filepath = os.path.join(savepath, file.name)
            if os.path.exists(filepath) and md5_value(filepath) == file.md5:
                file.local_path = filepath
                message.setFileDown(file, 100)
            else:
                success = await download(filepath, file.src, lambda p, s, t: message.setFileDown(file, p))
                if success:
                    if md5_value(filepath) != file.md5:
                        error = file.label + ' 文件校验码不一致'
                        break
                    else:
                        file.local_path = filepath

                    """ if os.path.getsize(filepath) != file.size:
                        error = file.label + ' 文件尺寸不一致'
                        break """
                else:
                    error = file.label+' 下载失败'
                    break

        if error:
            if(message.retried < message.download_retry):
                message.retried += 1
                print(f'\n\t{error}，5秒后尝试第{message.retried+1}次重试')
                await asyncio.sleep(5)
                return await self.download(message)
            print(f'\n\t{error}')
            await self.update(message._id, 'completed', 'error', message=error)
            return False
        else:
            return True

    async def web_publish(self, message: Message):
        youban = Youban(message.category.platform, message.account)
        # 先登录
        res = await youban.login()
        if not res.success:
            print(f'\n\t{res.msg}')
            await self.update(message._id, 'completed', 'error', message=res.msg)
            return

        error = ''
        # 文件上传到cos
        for file in message.files:
            module = 'upload_cover' if file.accept == 'video/jpeg' else 'upload_video'
            res = await youban.web_get_token(module)

            if not res.success:
                error = res.msg
                break

            res = await to_cos(res.data, file.local_path, file.accept, lambda p, s, t: message.setFileUp(file, p))
            if not res.success:
                error = res.msg or '上传时未知错误'
                break

            if file.id == 0:
                message.video_cos = res.data
            elif file.id == 1:
                message.coverinfo_cos = res.data  # 是横版封面
            elif file.id == 2:
                message.coverinfo2_cos = res.data  # 是竖版封面
            elif file.id == 3:
                message.coverinfo1_cos = res.data  # 好像是分屏封面

        if error:
            print(f'\n\t{error}')
            await self.update(message._id, 'completed', 'error', title=message.value.title, message=error)
        else:
            try:
                Location = message.video_cos.get('Location')
                await self.update(message._id, 'publishing', 'uploaded', location=Location)
            except:
                pass

            payload = message.payload
            if message.category.platform == 'tp':
                payload['substance_cateid'] = '混剪'

            res = await youban.web_submit(message.files[0].local_path, payload)
            if res.success:
                print(f'\n\t\033[32m发布成功\033[0m')
                await self.update(message._id, 'completed', 'publish', title=message.value.title)
                self.remove_local_files(message)
            else:
                print(f'\n\t\033[31m{res.msg}\033[0m')
                await self.update(message._id, 'completed', 'error', message=res.msg)

    async def api_publish(self, message: Message):
        error = ''
        youban = Youban(message.category.platform, message.account)
        # 文件上传到cos
        for file in message.files:
            module = 'upload_cover' if file.accept == 'video/jpeg' else 'upload_video'
            res = await youban.api_get_token(module)

            if not res.success:
                error = res.msg
                break

            res = await to_cos(res.data, file.local_path, file.accept, lambda p, s, t: message.setFileUp(file, p))
            if not res.success:
                error = res.msg or '上传时未知错误'
                break

            if file.id == 0:
                message.video_cos = res.data
            elif file.id == 1:
                message.coverinfo_cos = res.data  # 是横版封面
            elif file.id == 2:
                message.coverinfo2_cos = res.data  # 是竖版封面
            elif file.id == 3:
                message.coverinfo1_cos = res.data  # 好像是分屏封面

        if error:
            print(f'\n\t{error}，暂停5秒后继续下一条')
            await self.update(message._id, 'completed', 'error', message=error)
            await asyncio.sleep(5)
        else:
            try:
                Location = message.video_cos.get('Location')
                await self.update(message._id, 'publishing', 'uploaded', location=Location)
            except:
                pass
            res = await youban.api_submit(message.payload)
            if res.success:
                print(f'\n\t\033[32m发布成功\033[0m')
                await self.update(message._id, 'completed', 'publish', title=message.value.title)
                self.remove_local_files(message)
            else:
                print(f'\n\t{res.msg}')
                await self.update(message._id, 'completed', 'error', message=res.msg)

    async def update(self, _id: str, status: str, childStatus: str, title: str = None, message: str = None, location: str = None):
        kwargs = dict(_id=_id, status=status, childStatus=childStatus)
        if title:
            kwargs['title'] = title
        if message:
            kwargs['message'] = message
        if location:
            kwargs['location'] = location
        await self.query('/publish/update', **kwargs)
        return Result(True, wait=0)

    async def query(self, url: str, **kwargs):
        sign, nonce = sign_dict(kwargs, self.appsecret)

        url = f'{self.host}{url}?appid={self.appid}&nonce={nonce}&sign={sign}&version={self.version}'

        headers = {'Content-Type': 'application/json'}

        response = requests.post(url, data=json.dumps(
            kwargs), headers=headers) if kwargs else requests.get(url, headers=headers)

        if(response.status_code == 200):
            value = response.json()

            res = Result(
                success=value.get('success'),
                msg=value.get('msg'),
                data=value.get('data'),
                wait=value.get('wait')
            )
            return res
        else:
            return Result(True, msg=f'{response.text},{response.status_code}', wait=5)


# 定义标志位
exit_flag2 = False

# 信号处理函数


def signal_handler(signum, frame):
    global exit_flag2
    exit_flag2 = True
    print("\nExiting...")


# 注册信号处理函数
signal.signal(signal.SIGINT, signal_handler)


async def run(version, host, appid, appsecret, workdir):

    print(
        f'\n\n《八戒来了之发布程序 - {version}》\n服务器：{host}\nAppId：{appid}\n工作目录：{workdir}\n')

    client = Publish(version, host, workdir, appid, appsecret)

    while not exit_flag2:
        try:
            res = await client.run()
        except Exception as e:
            # log.error(e,exc_info=True)
            print(f'\n\t{e}')
            await asyncio.sleep(5)
        else:
            if res.msg:
                print(f'\n{res.msg}')
            if not res.success:
                break

            if res.wait > 0:
                next = datetime.datetime.now()+datetime.timedelta(seconds=res.wait)
                print(
                    f'\r暂无任务，{res.wait}秒后({next.strftime("%H:%M:%S")})将再次读取...', end=" ")
                await asyncio.sleep(res.wait)

        if exit_flag2:
            break

    print('已退出程序')
    await client.query('/publish/exit')


def main():
    version, host, appid, appsecret = read()
    if not appid or not appsecret:
        return
    workdir = read_workdir('publish')
    if not workdir:
        return
    
    parser = argparse.ArgumentParser()
    parser.add_argument('-dir', type=str, default='')
    parser.add_argument('-appid', type=str, default='')
    parser.add_argument('-appsecret', type=str, default='')
    parser.add_argument('-host', type=str, default='')
    args = parser.parse_args()

    if args.host and args.host!=host:
        host = args.host
        save_config('server','host',host)

    if args.appid and args.appid!=appid:
        appid = args.appid
        save_config('server','appid',appid)

    if args.appsecret and args.appsecret!=appsecret:
        appsecret = args.appsecret
        save_config('server','appsecret',appsecret)
    
    if args.dir and args.dir!=workdir:
        if not os.path.exists(args.dir):
            print(f'{args.dir}目录不存在')
            return
        workdir = args.dir
        save_config('publish','workdir',workdir)

    if sys.version_info.minor >= 10:
        asyncio.run(run(version, host, appid, appsecret, workdir))
    else:
        loop = asyncio.get_event_loop()
        loop.run_until_complete(run(version, host, appid, appsecret, workdir))
        loop.close()



from typing import Any
from ._object import Object


class Result(Object):
    def __init__(self, success: bool = False, msg: str = None, data: Any = None, wait: int = 5, **kwargs) -> None:
        self.success = success
        self.msg = msg or ''
        self.data = data
        self.wait = wait or 0



import argparse
import asyncio
import datetime
import json
import logging
import os
import signal
import sys

import requests

from .config import read, read_workdir, save_config
from ._message import ExportMessage
from ._result import Result
from .utils import download, md5_value, sign_dict, ymd

log = logging.getLogger(__name__)

class Export:
    def __init__(self, version: str, host: str, workdir: str, appid: str, appsecret: str) -> None:
        self.do_action = ''
        self.version = version
        self.host = host
        self.workdir = workdir
        self.appid = appid
        self.appsecret = appsecret

    async def run(self) -> Result:
        res = await self.query('/publish/export?action=get')
        
        if(not res.success or not res.data):
            return res

        for data in res.data:
            try:
                await self.download(ExportMessage(**data))
            except Exception as e:
                print(str(e))

        return Result(True, wait=5)
        
    async def download(self, message: ExportMessage):
        message.console_title()

        savepath = os.path.join(self.workdir, ymd(), message.category)
        os.makedirs(savepath, exist_ok=True)

        def remove_file(path):
            if os.path.exists(path):
                os.remove(path)

        error = ''
        for file in message.files:
            filepath = os.path.join(savepath, file.name)
            if os.path.exists(filepath) and md5_value(filepath) != file.md5:
                file.local_path = filepath
                message.setFileDown(file, 100)
            else:
                success = await download(filepath, file.src, lambda p, s, t: message.setFileDown(file, p))
                if success:
                    if md5_value(filepath) != file.md5:
                        error = file.label + ' 文件尺寸不一致'
                        remove_file(filepath)
                        break
                    else:
                        file.local_path = filepath
                else:
                    error = file.label+' 下载失败'
                    remove_file(filepath)
                    break

        if error:
            if(message.retried < message.download_retry):
                message.retried += 1
                print(f'\n\t{error}，5秒后尝试第{message.retried+1}次重试')
                await asyncio.sleep(5)
                return await self.download(message)
            print(f'\n\t{error}')
            await self.query('/publish/export?action=update',_id=message._id, status='error', error=error)
            return False
        else:
            print(f'\n\t\033[32m导出完成\033[0m')
            await self.query('/publish/export?action=update',_id=message._id, status='success')
            return True

    async def query(self, url: str, **kwargs):
        sign, nonce = sign_dict(kwargs, self.appsecret)

        url = f'{self.host}{url}&appid={self.appid}&nonce={nonce}&sign={sign}&version={self.version}'

        headers = {'Content-Type': 'application/json'}

        response = requests.post(url, data=json.dumps(
            kwargs), headers=headers) if kwargs else requests.get(url, headers=headers)

        if(response.status_code == 200):
            value = response.json()

            res = Result(
                success=value.get('success'),
                msg=value.get('msg'),
                data=value.get('data'),
                wait=value.get('wait')
            )
            return res
        else:
            return Result(True, msg=f'{response.text},{response.status_code}', wait=5)
        

# 定义标志位
exit_flag1 = False

# 信号处理函数
def signal_handler(signum, frame):
    global exit_flag1
    exit_flag1 = True
    print("\nExiting...")


# 注册信号处理函数
signal.signal(signal.SIGINT, signal_handler)

async def run(version, host, appid, appsecret,workdir):

    print(f'\n\n《八戒来了之导出程序 - {version}》\n服务器：{host}\nAppId：{appid}\n工作目录：{workdir}\n')
    
    client = Export(version, host, workdir, appid, appsecret)
    
    while not exit_flag1:
        try:
            res = await client.run()
        except Exception as e:
            log.error(e,exc_info=True)
            print(f'\n\t{e}')
            await asyncio.sleep(5)
        else:
            if res.msg:
                print(f'\n{res.msg}')
            if not res.success:
                break
            if res.wait>0:
                next = datetime.datetime.now()+datetime.timedelta(seconds=res.wait)
                print(f'\r暂无任务，{res.wait}秒后({next.strftime("%H:%M:%S")})将再次读取...',end=" ")
                await asyncio.sleep(res.wait)
                
        if exit_flag1:
            break

    print('已退出程序')
    

def main():
    version, host, appid, appsecret = read()
    if not appid or not appsecret:
        return
    workdir = read_workdir('export')
    if not workdir:
        return
    
    parser = argparse.ArgumentParser()
    parser.add_argument('-dir', type=str, default='')
    parser.add_argument('-appid', type=str, default='')
    parser.add_argument('-appsecret', type=str, default='')
    parser.add_argument('-host', type=str, default='')
    args = parser.parse_args()

    if args.host and args.host!=host:
        host = args.host
        save_config('server','host',host)

    if args.appid and args.appid!=appid:
        appid = args.appid
        save_config('server','appid',appid)

    if args.appsecret and args.appsecret!=appsecret:
        appsecret = args.appsecret
        save_config('server','appsecret',appsecret)
    
    if args.dir and args.dir!=workdir:
        if not os.path.exists(args.dir):
            print(f'{args.dir}目录不存在')
            return
        workdir = args.dir
        save_config('export','workdir',workdir)

    if sys.version_info.minor >= 10:
        asyncio.run(run(version, host, appid, appsecret,workdir))
    else:
        loop = asyncio.get_event_loop()
        loop.run_until_complete(run(version, host, appid, appsecret,workdir))
        loop.close()


import math
from ._object import Object
from typing import Any, List, Union


class VideoValue(Object):
    def __init__(self, title: str, category: str, tag: List[str], source: int, activity: Union[int, None], **kwargs) -> None:
        self.title = title
        self.category = category
        self.tag = tag
        self.source = source
        self.activity = activity


class Category(Object):
    def __init__(self, alias: str, name: str, platform: str, mode: str, creator: str, **kwargs) -> None:
        self.alias = alias
        self.name = name
        self.platform = platform
        self.mode = mode
        self.creator = creator


class File(Object):
    local_path: str
    down_value: str
    up_value: str

    def __init__(self, id: int, label: str, md5: str, src: str, size: int, accept: str, name: str, **kwargs) -> None:
        self.id = id
        self.label = label
        self.md5 = md5
        self.src = src
        self.size = size
        self.accept = accept
        self.name = name
        self.local_path = ''
        self.down_value = '0'
        self.up_value = '0'

    @property
    def sizeStr(self):
        value = 0
        unit = ''
        if self.size < 1024:
            value = self.size
            unit = 'B'
        elif self.size < 1024*1024:
            value = self.size/1024
            unit = 'KB'
        elif self.size < 1024*1024*1024:
            value = self.size/1024/1024
            unit = 'MB'
        else:
            value = self.size/1024/1024/1024
            unit = 'GB'
        return f'{math.ceil(value*10)/10}{unit}'


class Account(Object):
    def __init__(self, _id: str, name: str, weight: str, mobile: str, password: str, app_id: str, app_token: str, **kwargs) -> None:
        self._id = _id
        self.name = name
        self.weight = weight
        self.mobile = mobile
        self.password = password
        self.app_id = app_id
        self.app_token = app_token


Creator = {'mix': '混剪', 'split': '拆条', 'explain': '解说', 'inventory': '盘点'}


class Message(Object):
    _id: str
    value: VideoValue
    category: Category
    files: List[File]
    account: Account
    video_cos: Any
    coverinfo_cos: Any
    coverinfo2_cos: Any
    coverinfo1_cos: Any
    download_retry: int
    retried: int

    def __init__(self, _id: str, value: dict, account: dict, category: dict, files: List[dict]) -> None:
        self._id = _id
        self.value = VideoValue(**value)
        self.category = Category(**category)
        self.files = [File(**file) for file in files]
        self.account = Account(**account)
        self.download_retry = 1
        self.retried = 0
        self.video_cos = None
        self.coverinfo_cos = None
        self.coverinfo1_cos = None
        self.coverinfo2_cos = None

    def setFileDown(self, file: File, value):
        file.down_value = f'{value:.1f}%'
        self.console_progress()

    def setFileUp(self, file: File, value):
        file.up_value = f'{value:.1f}%'
        self.console_progress()

    def console_title(self):
        caption = f'\n【{self.account.name}】【{self.category.mode}发布】{self.value.title}'
        if(self.value.activity):
            caption += '【活动】'

        output = ''
        for file in self.files:
            label = file.label+'('+file.sizeStr+')'
            output += f'{label:<30}'

        print(f'{caption}\n\t{output}')

    def console_progress(self):
        output2 = ''
        for file in self.files:
            output2 += f'↓{file.down_value:<15}'
            output2 += f'↑{file.up_value:<17}'
        
        print(f'\r\t{output2}', end=" ")

    @property
    def payload(self):
        return {
            'title': self.value.title,
            'cover': 0,
            'category': self.value.category,
            'source': self.value.source,
            'tag': self.value.tag,
            'inspiration_cateid': '',
            'substance_cateid': Creator.get(self.category.creator),
            'activity': self.value.activity,
            'desc': '',
            'video_cos': self.video_cos,
            'coverinfo_cos': self.coverinfo_cos,
            'coverinfo2_cos': self.coverinfo2_cos,
            'coverinfo1_cos': self.coverinfo1_cos
        }


class ExportMessage(Object):
    _id: str
    category: str
    title: str
    files: List[File]
    download_retry: int
    retried: int

    def __init__(self, _id: str, category: str, title: str, files: List[dict]) -> None:
        self._id = _id
        self.category = category
        self.files = [File(**file) for file in files]
        self.title = title
        self.download_retry = 1
        self.retried = 0

    def setFileDown(self, file: File, value):
        file.down_value = f'{value:.1f}%'
        self.console_progress()

    def setFileUp(self, file: File, value):
        file.up_value = f'{value:.1f}%'
        self.console_progress()

    def console_title(self):
        caption = f'\n【导出】{self.title}'

        output = ''
        for file in self.files:
            label = file.label+'('+file.sizeStr+')'
            output += f'{label:<30}'

        print(f'{caption}\n\t{output}')

    def console_progress(self):
        output2 = ''
        for file in self.files:
            output2 += f'↓{file.down_value:<15}'

        print(f'\r\t{output2}', end=" ")



import json
from typing import List

import requests
from fake_useragent import UserAgent
from requests.cookies import RequestsCookieJar

from ._message import Account
from ._object import Object
from ._result import Result

# https://gist.github.com/kazqvaizer/4cebebe5db654a414132809f9f88067b


def multipartify(data, parent_key=None, formatter: callable = None) -> dict:
    if formatter is None:
        def formatter(v): return (None, v)  # Multipart representation of value

    if type(data) is not dict:
        return {parent_key: formatter(data)}

    converted = []

    for key, value in data.items():
        current_key = key if parent_key is None else f"{parent_key}[{key}]"
        if type(value) is dict:
            converted.extend(multipartify(
                value, current_key, formatter).items())
        elif type(value) is list:
            for ind, list_value in enumerate(value):
                iter_key = f"{current_key}[{ind}]"
                converted.extend(multipartify(
                    list_value, iter_key, formatter).items())
        else:
            converted.append((current_key, formatter(value)))

    return dict(converted)


class VideoPayload(Object):
    def __init__(self, title: str, category: str, source: int, tag: List[str],
                 video_cos: dict,
                 coverinfo_cos: dict,
                 coverinfo2_cos: dict = None,
                 coverinfo1_cos: dict = None,
                 inspiration_cateid: str = None,
                 substance_cateid: str = None,
                 desc: str = None,
                 activity: int = None, **kwargs) -> None:
        super().__init__()
        self.title = title
        self.category = category
        self.source = source
        self.tag = tag
        self.video_cos = video_cos
        self.coverinfo_cos = coverinfo_cos
        self.coverinfo2_cos = coverinfo2_cos
        self.coverinfo1_cos = coverinfo1_cos
        self.inspiration_cateid = inspiration_cateid or ''
        self.substance_cateid = substance_cateid or ''
        self.desc = desc or ''
        self.activity = activity or ''

    def __format_cos(self, data: dict, is_video: bool = False):
        result = {'statusCode': 200}
        for key, value in data.items():
            if key == 'Location':
                Location: str = value.replace(
                    'http://', '').replace('https://', '')
                result['Location'] = Location
                if is_video:
                    arr = Location.split('.')
                    result['Bucket'] = arr[0]
                    result['Key'] = '/'.join(Location.split('/')[1:])
            elif key == 'ETag':
                result['ETag'] = f'"{value}"'
            elif key == 'x-cos-request-id':
                result['RequestId'] = value
            elif key == 'UploadId':
                result['UploadId'] = value

        """ if is_video:
            result['headers'] = {
                'content-type': 'application/xml', 'x-cos-request-id': result['RequestId']}
        else:
            result['headers'] = {
                'content-length': '0', 'etag': result['ETag'], 'x-cos-request-id': result['RequestId']} """
        return result

    def get_data(self, mode: str,platform:str):
        payload = {
            'title': self.title,
            'cover': 0,
            'category': self.category,
            'source': self.source,
            'tag': self.tag,
            'inspiration_cateid': self.inspiration_cateid,
            'activity': self.activity,
            'desc': self.desc,
            'video_cos': self.__format_cos(self.video_cos),
            'coverinfo_cos': self.__format_cos(self.coverinfo_cos, False)
        }
        if(platform=='tp'):
            payload['substance_cateid']=self.substance_cateid
            
        if self.coverinfo2_cos:
            payload['coverinfo2_cos'] = self.__format_cos(
                self.coverinfo2_cos, False)
        if self.coverinfo1_cos:
            payload['coverinfo1_cos'] = self.__format_cos(
                self.coverinfo1_cos, False)

        return payload


class Youban(Object):
    platform: str
    account: Account
    userAgent: str
    cookies: RequestsCookieJar
    xsrfToken: str

    def __init__(self, platform: str, account: Account) -> None:
        self.platform = platform
        self.account = account
        self.BASE_URL = f"https://{self.platform}.xiaozhuyouban.com"
        self.userAgent = UserAgent().random
        self.cookies = None
        self.xsrfToken = ''

    async def api_get_token(self, module: str):
        """
            module = upload_video | upload_cover
        """
        try:
            url = f'https://{self.platform}.xiaozhuyouban.com/video/publish/uploadtoken?app_id={self.account.app_id}&app_token={self.account.app_token}&module={module}'
            response = requests.get(url)
            if response.status_code == 200:
                data = response.json()
                if data.get('code') == 0 and data.get('data') and data['data'].get('cos'):
                    return Result(True, data=data['data'].get('cos'))
                else:
                    return Result(msg=data.get('message') or '获取上传token失败')
            else:
                return Result(msg='获取上传token失败')
        except Exception as e:
            return Result(msg=str(e))

    async def api_submit(self,payload: dict):
        data = VideoPayload(**payload).get_data('api',self.platform)
        data['app_id'] = self.account.app_id
        data['app_token'] = self.account.app_token
        
        headers = {'Content-Type': 'application/json'}
        url = f'https://{self.platform}.xiaozhuyouban.com/video/publish'

        response = requests.post(url, data=json.dumps(data), headers=headers)

        if(response.status_code == 200):
            value = response.json()
            code = value.get('code')
            message = value.get('message') or 'api发布失败'
            return Result(code == 0, message, dict(code=code, message=message))
        else:
            message = f'发布请求异常({response.status_code})'
            return Result(False, message, dict(code=-1, message=message))

    async def login(self):
        r = requests.get(self.BASE_URL)
        self.xsrfToken = r.cookies.get("XSRF-TOKEN")

        headers = {
            'authority': f'{self.platform}.xiaozhuyouban.com',
            'accept': '*/*',
            'accept-language': 'zh-CN,zh;q=0.9,en;q=0.8,zh-TW;q=0.7',
            'content-type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'origin': self.BASE_URL,
            'referer': f'{self.BASE_URL}/',
            'user-agent': self.userAgent,
            'x-requested-with': 'XMLHttpRequest'
        }

        url = f"https://{self.platform}.xiaozhuyouban.com/signin"
        r = requests.post(
            url,
            data={"xsrfToken": self.xsrfToken,
                  "mobile": self.account.mobile, "password": self.account.password, "agree": "on"},
            cookies=r.cookies,
            headers=headers
        )
        if r.status_code == 200:
            data = r.json() or {'code': -1}
            code = data.get('code')
            message = data.get('message') or '登录未知错误'
            if code == 0:
                r.cookies.set('__root_domain_v', '.xiaozhuyouban.com')
                self.cookies = r.cookies
                return Result(True)
            else:
                action = 0
                if '爬虫' in message:
                    action = 401  # 全部停止
                return Result(msg=message)
        else:
            return Result(msg=f'登录失败：{r.status_code}')

    async def web_get_token(self, module: str):
        url = f'https://{self.platform}.xiaozhuyouban.com/storage/token'

        headers = {
            'authority': f'{self.platform}.xiaozhuyouban.com',
            'accept': '*/*',
            'accept-language': 'zh-CN,zh;q=0.9,en;q=0.8,zh-TW;q=0.7',
            'content-type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'origin': self.BASE_URL,
            'referer': f'{self.BASE_URL}/content/publish',
            'user-agent': self.userAgent,
            'x-requested-with': 'XMLHttpRequest',
            'x-csrf-token': self.xsrfToken
        }

        r = requests.post(
            url,
            data={'module': module},
            cookies=self.cookies,
            headers=headers
        )

        if r.status_code == 200:
            data = r.json()
            if data.get('code') == 0 and data.get('data') and data['data'].get('cos'):
                return Result(True, data=data['data'].get('cos'))
            else:
                return Result(msg=data.get('message') or '获取上传token失败')
        else:
            return Result(msg=f'获取上传token失败')

    async def web_submit(self, filepath: str, payload: dict):
        data = VideoPayload(**payload).get_data('web',self.platform)
        
        headers = {
            'authority': f'{self.platform}.xiaozhuyouban.com',
            'accept': '*/*',
            'accept-language': 'zh-CN,zh;q=0.9,en;q=0.8,zh-TW;q=0.7',
            'origin': self.BASE_URL,
            'referer': f'https://{self.platform}.xiaozhuyouban.com/content/publish',
            'user-agent': self.userAgent,
            'x-requested-with': 'XMLHttpRequest',
            'x-csrf-token': self.xsrfToken,
        }

        data = multipartify(data)
        data["coverfile"] = ('cover.jpg', open(filepath, 'rb'), 'image/jpeg')

        response = requests.post(
            f'https://{self.platform}.xiaozhuyouban.com/content/publish',
            files=data, headers=headers, cookies=self.cookies
        )

        if(response.status_code == 200):
            value = response.json()
            code = value.get('code')
            message = value.get('message') or 'web发布失败'
            return Result(code == 0, message, dict(code=code, message=message))
        else:
            message = f'发布请求异常({response.status_code})'
            return Result(False, message, dict(code=-1, message=message))



import datetime
import hashlib
import hmac
import random
import socket
from contextlib import closing
from typing import Callable

import requests
from qcloud_cos import CosConfig, CosS3Client
from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor

from ._result import Result


def ymd():
    return datetime.datetime.now().strftime('%Y%m%d')


def sign_dict(data, secret):
    """
    对dict数据进行签名
    :param data: 需要签名的数据
    :param secret: 密钥
    :return: 签名后的字符串
    """
    # 对key键进行降序排列
    sorted_data = sorted(data.items(), key=lambda x: x[0], reverse=True)
    # 将排列后的dict转换成字符串
    data_str = '&'.join([f'{k}={v}' for k, v in sorted_data])
    # 随机生成一个字符串作为盐值
    nonce = str(random.random())
    # 拼接数据和盐值
    message = nonce + data_str
    # 使用HMAC对数据进行签名
    sign = hmac.new(secret.encode(), message.encode(),
                    hashlib.sha256).hexdigest()
    # 返回签名后的字符串和盐值
    return sign, nonce


async def download(savepath: str, src: str, callback: Callable[[int, int, int], None]):
    try:
        with closing(requests.get(src, stream=True)) as response:
            chunk_size = 1024  # 单次请求最大值
            total = int(response.headers['content-length'])  # 内容体总大小
            size = 0
            with open(savepath, "wb") as file:
                for data in response.iter_content(chunk_size=chunk_size):
                    file.write(data)
                    size = size + len(data)
                    progress = (size / total) * 100
                    callback(progress, size, total)
    except:
        return False
    else:
        return True
    

async def to_cos(cos: dict, filepath: str, accept: str, callback):
    def cb(monitor):
        progress = (monitor.bytes_read / monitor.len) * 100
        callback(progress, monitor.bytes_read, monitor.len)

    bucket = cos.get('bucket')
    key = cos.get('filekey')
    region = cos.get('region')
    StartTime = cos.get('startTime')
    expiredTime = cos.get('expiredTime')
    credentials = cos.get('credentials') or []
    secret_id = credentials.get('tmpSecretId')
    secret_key = credentials.get('tmpSecretKey')
    token = credentials.get('sessionToken')
    url = f'https://{bucket}.cos.ap-shanghai.myqcloud.com{key}'

    config = CosConfig(Region=region, SecretId=secret_id,
                       SecretKey=secret_key, Token=token)
    client = CosS3Client(config)

    def cb(monitor):
        progress = (monitor.bytes_read / monitor.len) * 100
        callback(progress, monitor.bytes_read, monitor.len)

    fields = {
        'key': key,
        'Content-Type': accept,
        'file': (key, open(filepath, 'rb'), accept)
    }
    e = MultipartEncoder(fields)
    m = MultipartEncoderMonitor(e, cb)
    headers = {
        'Host': f'{bucket}.cos.ap-shanghai.myqcloud.com',
        'Content-Type': m.content_type,
        'x-cos-security-token': token
    }
    headers['Authorization'] = client.get_auth(
        "POST", bucket, key, Expired=expiredTime-StartTime)

    try:
        r = requests.post(url, data=m, headers=headers)

        if(r.status_code == 200 or r.status_code == 204):
            return Result(
                success=True,
                data={
                    'ETag': r.headers.get('ETag').replace('"', ''),
                    'Location': r.headers.get('Location'),
                    'x-cos-request-id': r.headers.get('x-cos-request-id')
                }
            )
        else:
            return Result(msg=r.text)
    except Exception as ex:
        return Result(msg=str(ex), data=ex)

#计算文件md5
def md5_value(filename):
    m = hashlib.md5()
    with open(filename, 'rb') as f:
        while True:
            data = f.read(8192)
            if not data:
                break
            m.update(data)
    return m.hexdigest()

def get_host_ip():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        address = ("8.8.8.8", 80)
        s.connect(address)
        ip = s.getsockname()[0]
    except:
        ip = '127.0.0.1'
    finally:
        s.close()
        
    return ip

def port_is_used(ip,port):
    """
    判断端口是否可用
    """
    s = None
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(1)
        s.connect((ip, port))
        return True
    except Exception as e:
        return False
    finally:
        if s:
            s.close()
            


from flask import Flask, request, jsonify, send_from_directory
from flask_cors import cross_origin, CORS
import os
import datetime
import argparse
from .config import read_web, save_config

BASE_URL = ''

app = Flask(__name__)
CORS(app)


@app.route('/ping')
@cross_origin(origins='*')
def ping():
    return jsonify({'success': True})


@app.route('/static/<path:filename>')
def serve_static(filename):
    return send_from_directory(app.static_folder, filename)


@app.route('/', methods=['GET', 'POST'])
@cross_origin(origins='*')
def upload():
    if request.method == 'GET':
        return '''
            <!doctype html>
            <title>局域网存储</title>
            <h1>短视频工作平台之局域网存储</h1>
        '''

    key = request.form.get('key')
    f = request.files['file']
    if not f and not key:
        return jsonify({'success': False, 'msg': '缺少必要参数'})

    isVideo = f.mimetype.startswith('video/mp4')
    isImage = f.mimetype.startswith('image/')
    if not isVideo and not isImage:
        return jsonify({'success': False, 'msg': '必须上传mp4或图片文件'})

    save_dir = os.path.join(
        app.static_folder, datetime.date.today().strftime('%Y%m%d'))
    # 如果目录不存在则创建
    if not os.path.exists(save_dir):
        os.makedirs(save_dir)

    # 保存文件
    save_path = os.path.join(save_dir, key)
    f.save(save_path)
    url_path = save_path.replace(app.static_folder, '').replace('\\', '/')
    # 返回文件路径
    return jsonify({'success': True, 'msg': f"{BASE_URL}/static{url_path}"})


def run(host: str, port: int, workdir: str):
    global BASE_URL
    BASE_URL = f'http://{host}:{port}'
    app.static_folder = workdir
    app.run(host=host, port=port, threaded=True)


def main():

    version, host, port, workdir = read_web()
    if not port or not workdir:
        return
    parser = argparse.ArgumentParser()
    parser.add_argument('-port', type=int, default=0)
    parser.add_argument('-dir', type=str, default='')

    args = parser.parse_args()
    if args.port and args.port != port:
        port = args.port
        save_config('web', 'port', port)

    if args.dir and args.dir != workdir:
        if not os.path.exists(args.dir):
            print(f'{args.dir}目录不存在')
            return
        workdir = args.dir
        save_config('web', 'workdir', workdir)

    print(f'\n\n《八戒来了之上传服务 - {version}》\nhost：{host}:{port}\n工作目录：{workdir}\n')
    run(host, port, workdir)



from setuptools import setup,find_packages
from dcvideo.config import VERSION

requires = [
    "requests==2.28.1",
    "requests_toolbelt==0.9.1",
    "cos_python_sdk_v5==1.9.21",
    "fake_useragent==1.1.1",
    "flask==2.2.3",
    "flask-cors==3.0.10",
]


setup(
    name='dcvideo', 
    version=VERSION,
    description='', 
    long_description='', 
    author='netfere', 
    author_email='netfere@gmail.com',
    url='https://www.shortvideo.work',
    packages=['dcvideo'],    # 项目需要的包
    python_requires=">=3.7",  # Python版本依赖
    install_requires=requires,  # 第三方库依赖
    entry_points={
        'console_scripts':[
            'video-pub = dcvideo.publish:main',
            'video-exp = dcvideo.export:main',
            'video-web = dcvideo.web:main'
        ]
    },
)